from django.conf import settings

JWT_ALGORITHM = getattr(settings, "JWT_ALGORITHM", "ES256")
JWT_AUDIENCE = getattr(settings, "JWT_AUDIENCE", ["account", "broker"])
JWT_CERTS_URL = getattr(settings, "JWT_CERTS_URL", None)

# key from KeyCloak; value is user model
JWT_USER_MAPPING = getattr(
    settings,
    "JWT_USER_MAPPING",
    {
        "given_name": "first_name",
        "family_name": "last_name",
        "name": "username",
    },
)
JWT_USER_UID = getattr(settings, "JWT_USER_UID", "kc_id")
JWT_USER_DEFAULTS = getattr(
    settings,
    "JWT_USER_DEFAULTS",
    {
        "is_active": True,
    },
)
JWT_USER_ON_CREATE = getattr(
    settings,
    "JWT_USER_ON_CREATE",
    None,
)
JWT_USER_ON_UPDATE = getattr(
    settings,
    "JWT_USER_ON_UPDATE",
    None,
)



import json

import jwt
import requests
from jwt.algorithms import ECAlgorithm, RSAAlgorithm

from django_jwt import settings


def _get_public_keys() -> dict:
    certs_data = requests.get(settings.JWT_CERTS_URL).json()
    public_keys = {}
    for key_data in certs_data["keys"]:
        if key_data["kty"] == "RSA":
            public_keys[key_data["alg"]] = RSAAlgorithm.from_jwk(json.dumps(key_data))
        elif key_data["kty"] == "EC":
            public_keys[key_data["alg"]] = ECAlgorithm.from_jwk(json.dumps(key_data))
    return public_keys


class KCVerifier:
    _public_key = None

    @property
    def public_key(self) -> str:
        if self._public_key is None:
            keys = _get_public_keys()
            self._public_key = keys[settings.JWT_ALGORITHM]
        return self._public_key

    def verify_token(self, token: str) -> dict:
        return jwt.decode(
            token,
            key=self.public_key,
            algorithms=[settings.JWT_ALGORITHM],
            audience=settings.JWT_AUDIENCE,
        )


verifier = KCVerifier()



from django.contrib.auth import get_user_model
from django.http.request import HttpRequest

from django_jwt import settings


class UserHandler:
    def __init__(self, payload: dict, request: HttpRequest):
        # auth0_id should be available if auth0 added in Client Scopes in KeyCloak admin
        self.kwargs = settings.JWT_USER_DEFAULTS.copy()
        self.kwargs.update(
            {ca_key: payload[kc_key] for kc_key, ca_key in settings.JWT_USER_MAPPING.items() if kc_key in payload}
        )
        self.kwargs["email"] = payload["email"].lower()
        self.kwargs[settings.JWT_USER_UID] = payload.get("auth0_id", payload["sub"])

        self.on_create = settings.JWT_USER_ON_CREATE
        self.on_update = settings.JWT_USER_ON_UPDATE
        self.request = request

    def _update_user(self, user):
        """Update user fields if they are changed in KeyCloak"""

        is_changed = False
        for key, val in self.kwargs.items():
            if getattr(user, key) != val:
                setattr(user, key, val)
                is_changed = True
        if is_changed:
            user.save(update_fields=self.kwargs.keys())

    def get_user(self):
        """
        Get user from database by kc_id or email.
        If user is not found, create new user.
        Update user fields if they are changed in KeyCloak.
        """
        model = get_user_model()
        try:
            user = model.objects.get(**{settings.JWT_USER_UID: self.kwargs[settings.JWT_USER_UID]})
        except model.DoesNotExist:
            try:
                user = model.objects.get(email=self.kwargs["email"])
            except model.DoesNotExist:
                user = model.objects.create(**self.kwargs)
                if self.on_create:
                    self.on_create(user, self.request)
                return user

        self._update_user(user)
        if self.on_update:
            self.on_update(user, self.request)
        return user
